use dotenvy::dotenv;
use entity;
use sea_orm::{ActiveModelTrait, Database, DatabaseConnection, DbConn, DbErr, Set};
use std::env;
use std::io::stdin;

pub async fn establish_connection() -> DatabaseConnection {
    dotenv().ok();
    let database_url = env::var("DATABASE_URL").expect("DATABASE_URL must be set");
    let db = Database::connect(&database_url).await;
    db.unwrap_or_else(|_| panic!("Error connecting to {}", database_url))
}

pub async fn create_user(conn: &DbConn, username: &str) -> Result<entity::user::Model, DbErr> {
    let user = entity::user::ActiveModel {
        username: Set(username.to_owned()),
        ..Default::default()
    };
    let user = user.insert(conn).await?;
    Ok(user)
}

#[tokio::main]
async fn main() {
    let connection = establish_connection().await;

    let mut username = String::new();

    println!("Enter username for new user:");
    stdin().read_line(&mut username).unwrap();
    let username = username.trim_end(); // Remove the trailing newline

    let user = create_user(&connection, username)
        .await
        .expect("Error creating user");
    println!("\nSaved user {} with id {}", username, user.id);
}
